import jpegtran_wrap

class transformer( object ):
    """Out base class for applying lossless transformation to an image."""
    
    def __init__( self, **kwargs ):
        """
        kwargs define the way the transformation should be done:
            progressive = True/False (True)
            optimize    = True/False (False)
            trim        = True/False (False)
            grayscale   = True/False (False)
            copy        = 'none', 'comments', 'all' (None = 'none')
            maxmemory   = an integer (None)
        """
        
        self.progressive = True
        self.optimize = False
        self.trim = False
        self.grayscale = False
        
        self.copy = None
        self.maxmemory = None

        self.__dict__.update( kwargs )

        # this is argv[0] in the original C version
        self.cmd = [ 'dummy' ]
        
        if self.progressive:
            self.cmd += [ '-progressive' ]
        if self.optimize:
            self.cmd += [ '-optimize' ]
        if self.trim:
            self.cmd += [ '-trim' ]
        if self.grayscale:
            self.cmd += [ '-grayscale' ]

        if self.copy is not None:
            self.cmd += [ '-copy', self.copy ]
        if self.maxmemory is not None:
            self.cmd += [ '-maxmemory', self.maxmemory ]

    def _exec( self, cmd ):

        if jpegtran_wrap.transform( cmd ) == 0:
            raise JpegtranError( cmd )

    def rotate( self, angle, input, output ):
        """Angle can be 90, 180, 270."""

        mycmd = self.cmd + [ '-rotate', '%d' % angle, '-outfile', output, input ]
        self._exec( mycmd )

    def flip( self, how, input, output ):
        """How is either horizontal or vertical."""

        mycmd = self.cmd + [ '-flip', how, '-outfile', output, input ]
        self._exec( mycmd )

    def transpose( self, input, output ):

        mycmd = self.cmd + [ '-transpose', '-outfile', output, input ]
        self._exec( mycmd )

    def transverse( self, input, output ):

        mycmd = self.cmd + [ '-transverse', '-outfile', output, input ]
        self._exec( mycmd )

    def gray( self, input, output ):

        mycmd = self.cmd + [ '-grayscale', '-outfile', output, input ]
        self._exec( mycmd )

class JpegtranError( Exception ):
    def __init__( self, cmd ):
        self.cmd = cmd
    def __str__( self ):
        return "Something's wrong with your command: %s" % ' '.join( self.cmd[1:] )
